@@ -37,12 +37,11 @@ static struct option getopt_long_options[] = {
     { "daemonize", 0, NULL, 'd' },
 #endif
     { "edns-payload-size", 1, NULL, 'e' },
+    { "ephemeral-keys", 0, NULL, 'E' },
     { "help", 0, NULL, 'h' },
     { "resolvers-list", 1, NULL, 'L' },
     { "resolver-name", 1, NULL, 'R' },
-#ifndef _WIN32
     { "logfile", 1, NULL, 'l' },
-#endif
     { "loglevel", 1, NULL, 'm' },
     { "max-active-requests", 1, NULL, 'n' },
 #ifndef _WIN32
@@ -64,9 +63,9 @@ static struct option getopt_long_options[] = {
     { NULL, 0, NULL, 0 }
 };
 #ifndef _WIN32
-static const char *getopt_options = "a:de:hk:L:l:m:n:p:r:R:t:u:N:TVX";
+static const char *getopt_options = "a:de:Ehk:L:l:m:n:p:r:R:t:u:N:TVX";
 #else
-static const char *getopt_options = "a:e:hk:L:m:n:r:R:t:u:N:TVX";
+static const char *getopt_options = "a:e:Ehk:L:l:m:n:r:R:t:u:N:TVX";
 #endif
 
 #ifndef DEFAULT_CONNECTIONS_COUNT_MAX
@@ -109,7 +108,7 @@ void options_init_with_default(AppContext * const app_context,
     proxy_context->connections_count_max = DEFAULT_CONNECTIONS_COUNT_MAX;
     proxy_context->edns_payload_size = (size_t) DNS_DEFAULT_EDNS_PAYLOAD_SIZE;
     proxy_context->local_ip = "127.0.0.1:53";
-    proxy_context->log_fd = -1;
+    proxy_context->log_fp = NULL;
     proxy_context->log_file = NULL;
     proxy_context->pid_file = NULL;
     proxy_context->resolvers_list = DEFAULT_RESOLVERS_LIST;
@@ -126,6 +125,7 @@ void options_init_with_default(AppContext * const app_context,
     proxy_context->test_cert_margin = (time_t) -1;
     proxy_context->test_only = 0;
     proxy_context->tcp_only = 0;
+    proxy_context->ephemeral_keys = 0;
 }
 
 static int
@@ -199,6 +199,9 @@ options_parse_resolver(ProxyContext * const proxy_context,
                        char * const * const headers, const size_t headers_count,
                        char * const * const cols, const size_t cols_count)
 {
+    const char *dnssec;
+    const char *namecoin;
+    const char *nologs;
     const char *provider_name;
     const char *provider_publickey_s;
     const char *resolver_ip;
@@ -235,6 +238,38 @@ options_parse_resolver(ProxyContext * const proxy_context,
                resolver_name);
         return -1;
     }
+    dnssec = options_get_col(headers, headers_count,
+                             cols, cols_count, "DNSSEC validation");
+    if (dnssec != NULL && strcasecmp(dnssec, "yes") != 0) {
+        logger(proxy_context, LOG_INFO,
+               "- [%s] does not support DNS Security Extensions",
+               resolver_name);
+    } else {
+        logger(proxy_context, LOG_INFO,
+               "+ DNS Security Extensions are supported");
+    }
+    namecoin = options_get_col(headers, headers_count,
+                               cols, cols_count, "Namecoin");
+    if (namecoin != NULL && strcasecmp(namecoin, "yes") != 0) {
+        logger(proxy_context, LOG_INFO,
+               "- [%s] does not support Namecoin domains",
+               resolver_name);
+    } else {
+        logger(proxy_context, LOG_INFO,
+               "+ Namecoin domains can be resolved");
+    }
+    nologs = options_get_col(headers, headers_count,
+                             cols, cols_count, "No logs");
+    if (nologs != NULL && strcasecmp(nologs, "no") == 0) {
+        logger(proxy_context, LOG_WARNING,
+               "- [%s] logs your activity - "
+               "a different provider might be better a choice if privacy is a concern",
+               resolver_name);
+    } else {
+        logger(proxy_context, LOG_INFO,
+               "+ Provider supposedly doesn't keep logs");
+    }
+
     proxy_context->provider_name = strdup(provider_name);
     proxy_context->provider_publickey_s = strdup(provider_publickey_s);
     proxy_context->resolver_ip = strdup(resolver_ip);
@@ -378,13 +413,11 @@ options_apply(ProxyContext * const proxy_context)
     }
 #endif
     if (proxy_context->log_file != NULL &&
-        (proxy_context->log_fd = open(proxy_context->log_file,
-                                      O_WRONLY | O_APPEND | O_CREAT,
-                                      (mode_t) 0600)) == -1) {
+        (proxy_context->log_fp = fopen(proxy_context->log_file, "a")) == NULL) {
         logger_error(proxy_context, "Unable to open log file");
         exit(1);
     }
-    if (proxy_context->log_fd == -1 && proxy_context->daemonize) {
+    if (proxy_context->log_fp == NULL && proxy_context->daemonize) {
         logger_open_syslog(proxy_context);
     }
     return 0;
@@ -423,9 +456,10 @@ options_parse(AppContext * const app_context,
             }
             if (edns_payload_size <= DNS_MAX_PACKET_SIZE_UDP_NO_EDNS_SEND) {
                 proxy_context->edns_payload_size = (size_t) 0U;
+                proxy_context->udp_max_size = DNS_MAX_PACKET_SIZE_UDP_NO_EDNS_SEND;
             } else {
                 proxy_context->edns_payload_size = (size_t) edns_payload_size;
-                assert(proxy_context->udp_max_size ==
+                assert(proxy_context->udp_max_size >=
                        DNS_MAX_PACKET_SIZE_UDP_NO_EDNS_SEND);
                 if (proxy_context->edns_payload_size > DNS_MAX_PACKET_SIZE_UDP_NO_EDNS_SEND) {
                     proxy_context->udp_max_size =
@@ -434,6 +468,9 @@ options_parse(AppContext * const app_context,
             }
             break;
         }
+        case 'E':
+            proxy_context->ephemeral_keys = 1;
+            break;
         case 'h':
             options_usage();
             exit(0);
